@@ -1,13 +1,21 @@
 import chess
 import numpy as np
 import tensorflow as tf
-from .training_tf import load_model,encode
+from .training_tf import load_model
 
+def encode(board,encoding):
+    b=str(board).replace(' ','').split('\n')
+    a=np.zeros([8,8,len(encoding['.'])])
+    for i,row in enumerate(b):
+        for j,val in enumerate(row):
+            a[i,j,:]=encoding[val]
+    return a
 
 class agent():
 
     def __init__(self,model,path_model):
-
+        physical_devices = tf.config.list_physical_devices('GPU')
+        print(physical_devices)
         self.encoding,self.history=load_model(model,path_model)
         self.model=model
         self.channels=len(self.encoding['.'])
@@ -31,6 +39,16 @@ class agent():
             print(f'nodo terminal, resultado: {board.result()}')
             return None
 
+    def get_move_values_single(self,board):
+        t_moves=np.zeros([1,8,8,self.channels],dtype=np.float32)
+        t_moves[0,:]=encode(board,self.encoding)
+        score=self.model(t_moves)
+        score=tf.nn.softmax(score,1)
+        score=score.numpy()
+        score = np.squeeze(score,0)
+        return score
+    
+
 
     def select_move(self,board):
         moves,values=self.get_move_values(board)
