@@ -39,15 +39,16 @@ def fitting(start=0,
             y_test=None,
             model=None,
             optimizer=None,
-            batch_size=128,
             lr=0.1,
             loss_fn=None,
-            save_name = 'model',
+            batch_size=128,
             encoding=None,
+            save_name = 'model',
             load_name = None,
             shuffle_train=True,
             shuffle_test=False,
-            download_models=False):
+            download_models=False,
+            num_workers=tf.data.AUTOTUNE):
 
     history={'train':{'acc':[],'loss':[]}, 'test':{'acc':[],'loss':[]} }
 
@@ -60,16 +61,16 @@ def fitting(start=0,
         encoding,history,start = load_model(model,load_name,training=True)
 
     
-    train_loader=data_loader( x_data = x_train,y_data=y_train,batch_size=batch_size,shuffle=True ,encoding = encoding )
-    test_loader = None
-    if x_test is not None and y_test is not None:
-        test_loader=data_loader( x_data = x_test,y_data=y_test,batch_size=batch_size,shuffle=False , encoding = encoding )
-
+    train_loader=data_loader( x_data = x_train,y_data=y_train,batch_size=batch_size,shuffle=True ,encoding = encoding,num_workers=num_workers)
     len_train_loader=len(train_loader)
-    len_test_loader=len(test_loader)
     percent_train = len_train_loader//100
-    percent_test = len_test_loader//100
-
+    test = False
+    if x_test is not None and y_test is not None:
+        test_loader=data_loader( x_data = x_test,y_data=y_test,batch_size=batch_size,shuffle=False , encoding = encoding,num_workers=num_workers )
+        len_test_loader=len(test_loader)
+        percent_test = len_test_loader//100
+        test=True
+    
     start+=1
     NUM_EPOCHS = start+epochs
 
@@ -85,20 +86,18 @@ def fitting(start=0,
             if percent_train<10 or i%percent_train == 0:
                 print_r('Epoch: {:02}/{:02} | train progress: {:.0f}/100 | train loss:{:.4f} | train acc: {:.4f}           '
                     .format(epoch,NUM_EPOCHS-1,(i+1)*100/len_train_loader,train_loss.result(),train_accuracy.result()))
+
+        history['train']['acc'].append(train_accuracy.result())
+        history['train']['loss'].append(train_loss.result())
         
-        if test_loader is not None:
+        if test:
             for i,(batch_x, batch_y) in enumerate(test_loader):
                 test_step(model,batch_x,batch_y,loss_fn)
                 if percent_test<10 or i%percent_test == 0:
                     print_r('Epoch: {:02}/{:02} | test progress: {:.0f}/100 | test loss:{:.4f} | test acc: {:.4f}           '
                         .format(epoch,NUM_EPOCHS-1,(i+1)*100/len_test_loader,test_loss.result(),test_accuracy.result()))
-
-
-        #template = 'Epoch {:02d}{}, Tiempo: {:.1f}m Perdida: {:.6f}, Exactitud: {:.2f}, Perdida de prueba: {:.6f}, Exactitud de prueba: {:.2f}'
-        history['train']['acc']=train_accuracy.result()
-        history['train']['loss']=train_loss.result()
-        history['test']['acc']=test_accuracy.result()
-        history['test']['loss']=test_loss.result()
+            history['test']['acc'].append(test_accuracy.result())
+            history['test']['loss'].append(test_loss.result())
 
         elapsed_time = time.time() - start_time
         name="{}.{}.h5".format(save_name,epoch)
@@ -108,7 +107,7 @@ def fitting(start=0,
             from google.colab import files
             files.download(name)
 
-        if test_loader is not None:
+        if test:
             print_r('Epoch: {:02}/{:02} | time: {:.0f}s = {:.1f}m | train loss: {:.4f} | train acc: {:.4f} | test loss: {:.4f} | test acc: {:.4f}'
                 .format(epoch,NUM_EPOCHS-1,elapsed_time,elapsed_time/60,train_loss.result(),train_accuracy.result(),test_loss.result(),test_accuracy.result()))
         else:
@@ -116,19 +115,13 @@ def fitting(start=0,
                 .format(epoch,NUM_EPOCHS-1,elapsed_time,elapsed_time/60,train_loss.result(),train_accuracy.result()))
 
 
-
         train_loss.reset_states()
         train_accuracy.reset_states()
-        test_loss.reset_states()
-        test_accuracy.reset_states()
+        if test:
+            test_loss.reset_states()
+            test_accuracy.reset_states()
+
 
-def encode(board,encoding):
-    b=str(board).replace(' ','').split('\n')
-    a=np.zeros([8,8,len(encoding['.'])])
-    for i,row in enumerate(b):
-        for j,val in enumerate(row):
-            a[i,j,:]=encoding[val]
-    return a
 
 def save_model(model,history,encoding,epoch,name):
     save_pkl((model.hw,model.hb,history,encoding,epoch),name)
@@ -144,37 +137,36 @@ def load_model(model,name,training=False):
     else:
         return encoding,history
 
+def recode(x_in,keys,values):
+    to_return=np.zeros([x_in.shape[0],64,len(values[0])],dtype=np.float32)
+    for i,value in enumerate(values):
+        to_change=np.where(x_in==keys[i])
+        to_return[to_change[0],to_change[1],:]=value
+    return np.reshape(to_return,(-1,8,8,len(values[0])))
 
-def data_loader(x_data,y_data,batch_size,shuffle=True,encoding = None, seed=0):
-    class Wrapper():
-        def __init__(self,encoding):
-            self.keys = np.array([params.inter_map[i] for i in encoding.keys()],dtype=np.int8)
-            self.values = np.stack([value for value in encoding.values()],0).astype(np.float32)
-        def __call__(self,x_in,y_in):
-            x_in_encoded = tf.numpy_function(
-                Wrapper.recode,
-                inp=(x_in,self.keys,self.values),
-                Tout=(tf.float32)
-            )
-            return x_in_encoded,y_in
-            
-        @staticmethod
-        def recode(x_in,keys,values):
-            to_return=np.zeros([x_in.shape[0],64,len(values[0])],dtype=np.float32)
-            for i,value in enumerate(values):
-                to_change=np.where(x_in==keys[i])
-                to_return[to_change[0],to_change[1],:]=value
-            return np.reshape(to_return,(-1,8,8,len(values[0])))
+class Wrapper():
+    def __init__(self,encoding):
+        self.keys = np.array([params.inter_map[i] for i in encoding.keys()],dtype=np.int8)
+        self.values = np.stack([value for value in encoding.values()],0).astype(np.float32)
+    def __call__(self,x_in,y_in):
+        x_in_encoded = tf.numpy_function(
+            recode,
+            inp=(x_in,self.keys,self.values),
+            Tout=(tf.float32)
+        )
+        return x_in_encoded,y_in
 
+
+def data_loader(x_data,y_data,batch_size,shuffle=True,encoding = None, seed=0,num_workers=None):
+
+    dataset=tf.data.Dataset.from_tensor_slices((x_data,y_data))
     
-    x_data=tf.data.Dataset.from_tensor_slices(x_data)
-    y_data=tf.data.Dataset.from_tensor_slices(y_data.astype(np.float32))
-    
-    dataset=tf.data.Dataset.zip((x_data,y_data))
-    dataset = dataset.shuffle(buffer_size=len(dataset),seed=seed,reshuffle_each_iteration=shuffle)
+    if shuffle:
+        buffer_size=12800
+        dataset = dataset.shuffle(buffer_size=buffer_size,seed=seed,reshuffle_each_iteration=shuffle)
     dataset = dataset.batch(batch_size)
     if encoding is not None:
         wrapper = Wrapper(encoding)
-        dataset=dataset.map(wrapper,num_parallel_calls=tf.data.AUTOTUNE)
+        dataset=dataset.map(wrapper,num_parallel_calls=num_workers)
     return dataset
 
