@@ -31,7 +31,7 @@ def fitting(start=0,
     
     history = {"train": {"loss": [], "acc": []},"test": {"loss": [], "acc": []}}
     if load_name is not None:
-        model,optimizer,loss_fn,start,encoding,history=load_model(model,load_name,training=True)
+        model,optimizer,loss_fn,start,encoding,history=load_model(model,load_name,training=True,device=device)
     else:
         optimizer=optim(model.parameters(),lr=lr,momentum=mo)
 
@@ -140,16 +140,10 @@ def fitting(start=0,
             print_r('Epoch: {:02}/{:02} | time: {:.0f}s = {:.1f}m | train loss: {:.4f} | train acc: {:.4f}'
                 .format(epoch,NUM_EPOCHS-1,elapsed_time,elapsed_time/60,epoch_train_loss,epoch_train_acc))
 
-def encode(board,encoding):
-    b=str(board).replace(' ','').split('\n')
-    a=torch.zeros([len(encoding['.']),8,8],dtype=torch.float)
-    for i,row in enumerate(b):
-        for j,val in enumerate(row):
-            a[:,i,j]=encoding[val]
-    return a
+
 
 def load_model(model,filename,training=False):
-    checkpoint = torch.load(filename)
+    checkpoint = torch.load(filename,map_location='cpu')
     model.load_state_dict(checkpoint['model_state_dict'])
     encoding=checkpoint['encoding']
     if training:
